%--------------------------------------------------------------------------
% Updated version of FEM solver for Burgers equation
% Orginally from Zhu Wang
%--------------------------------------------------------------------------

clc; clear; close all

addpath ../FEM_FUN
addpath ../PLOT
addpath ../POST
addpath ../OTHER

%--------------------------------------------------------------------------
fprintf(1,'===== New run starts =====\n');
time_start = tic;
%========= user-defined paremeters
%--- specify test problem
opt.es     = 0;          % choose test problem %%just for numbering of the case
opt.rhs    = 0;          % choose equations & rhs
opt.bc     = 0;          % choose boundary conditions
opt.ic     = 3;          % choose initial conditions
opt.theta  = 1;          % choose theta value, if negative, semi-implicit
domain.interval = [0, 1];% spatial interval
elem_order = 2;          % choose the order of element
domain.h   = 1/2^9;      % choose mesh size on spatial space
domain.range = [0, 1];   % time range
domain.t   = 1e-4;       % choose time step on tempral space
domain.tsave = domain.t;
domain.calerr_skip = 1;  % skip some steps when calculate errors

opt.parm_e = 1e-3; % viscosity
%------- filtering option
filter.op =0;
filter.delta =0;%1e-4;
%--- set numerical discretization
n_gauss    = 5;              % Gaussian quadrature in the FE integration
%--- set output
fig_num    = 1;
fig_evo_p  = 0;
fig_evo    = 1;
cal_err    = 0;              % output error values to screen
save_data  = 1;              % choose whether saving data or not
%--- generate 1D mesh
[x,e_conn,bn_index] = fem1d_mesh(domain, elem_order);     % generate mesh
% x: X-coordinates; e_conn: element info
n_nodes     = size(x,1);
n_elements  = size(e_conn,1);
%--- specify test problem
[u,f,ux,B,Nonlfunc]  = burgers1d_fem_setting(opt,domain);
% u.func(func):  either u or unknown;    u.var(sym): symbolic variable of u.
% f.func(func):  either f or unknown;    f.var(sym): symbolic variable of f.
% ux.func(func): either du/dx or unknow; ux.var(sym):symbolic variable of ux.
% B.type(str): type of BC;
% B.value(cell): boundary value.
%--- specify boundary condition
[B.index, Len] = fem1d_bc(x,bn_index,B.criterion);
% B_index(array): nonzero elements on each column are associated with one BC.
% Len(array): value of each entry is the number of nonzeroes on each column
%                in B_index.
t_initial      = domain.range(1);
t_final        = domain.range(2);
bn             = fem1d_bc_eval(x, B.index, Len, B.value, B.type, t_initial);
% bn.dir(array): first column contains Dirichlet boundary nodes,
%                secound column contains boundary values at t_initial;
% bn.neu(array): Neumann BC; bn.rob(array): Robin BC; bn.fre(array): free BC.
nodes_all = 1:n_nodes;
if isempty(bn.dir) == 1
    nodes_nondir = nodes_all;
else
    nodes_nondir = setdiff(nodes_all, bn.dir(:,1));
end
% nodes_nondir: non-Dirichlet nodes including both interior and non-Dirichlet boundary nodes
%--- specify initial condition
if opt.ic == 3
    u0 = burgers1d_step_ic(x);
elseif opt.ic == 4
    u0 = 8*opt.parm_e*pi.*sin(pi.*x)./(5+4.*cos(pi.*x));
else
    u0 = fem1d_ic(opt.ic,x,bn,u,nodes_nondir,t_initial);
end
% u0(array): initial condition vector.
%--- set file names
Folder_save = '../../FEM_Data/';
Filename0   = ['Burgers1D_FEM_case_',num2str(opt.es),num2str(opt.rhs),...
    '_P', num2str(elem_order), '_h', num2filename(domain.h),...
    '_nu',num2str(num2filename(opt.parm_e)),...
    '_dt', num2str(num2filename(domain.t)),...
    '_cta', num2str(num2filename(opt.theta)), '_'];
File_shape  = [Folder_save, Filename0];
File_smm    = [Folder_save, Filename0, 'FEIC_SMM'];
File_dns    = [Folder_save, Filename0,'dns'];
File_fig    = ['../Figures/', Filename0, 'dns'];
%--- integration preparation
File_shape = fem1d_shape_gen(x,e_conn,n_gauss,File_shape);   % generate and store shape function info.
% File_shape: data which stores shape function info.
[Mass, S, P]  = fem1d_assemble_bilinear(x,e_conn,n_gauss,File_shape,File_smm);  % generate mass and stiffness
% Mass(array): mass matrix; S1+S2(array): stiffness matrix; P: (hx, h)
%--- initialization of variables
n_save     = fix((t_final-t_initial)/domain.tsave)+1;
u_save     = zeros(n_nodes,n_save); u_exact = u_save;
u0(2:end-1) = (Mass(2:end-1,2:end-1)+filter.delta.^2*S(2:end-1,2:end-1))\(Mass(2:end-1,2:end-1)*u0(2:end-1));
u_save(:,1)= u0; % @ t=t_initial
u_exact(:,1)= u0;
count_s    = 1;
%--- calculate FE load vector
f_time_dep = sum(strcmp('t',f.var));% determin if problem is time-dependent
f_FE   = fem1d_load(x,e_conn,n_gauss,f,t_initial,File_shape);% FE load at t(n)
f_BE   = f_FE;                        % initial FE load at t(n+1)
%---- convert string to function handle
% BC_handle = str2func(B.value{1}); %for B.C.
if opt.rhs~=0                    %for nonlinear functions
    ker_nonlfunc_grad= str2func(Nonlfunc.grad_str);
    ker_nonlfunc = str2func(Nonlfunc.str);
else
    ker_nonlfunc_grad = [];
    ker_nonlfunc = [];
end
%____________________________% numerical simulation: time integrations
for i_t = t_initial: domain.t: t_final-domain.t
    time = i_t+ domain.t;
    if f_time_dep~=0         % if time-dependent then refresh FE load at t(n+1)
        f_BE   = fem1d_load(x,e_conn,n_gauss,f,time,File_shape);
    end
    %     Neu_FE = FEM_function_value(BC_handle,'t',0,i_t);  % Neumann BC at x=0 when t= t(n)
    %     Neu_BE = FEM_function_value(BC_handle,'t',0,time);% Neumann BC at x=0 when t= t(n+1)
    %     Neu_BE = (5e+4)*time^3*exp(-15*time);% Neumann BC at x=0 when t= t(n+1)
    if opt.theta<0
        %- semi-implicit method
        [U1] = Burgers1D_FEM_semi...
            (opt,domain,x,e_conn,n_gauss,Mass,S,P, ...
            u0,f_FE,f_BE,File_shape,nodes_nondir,bn,...
            ker_nonlfunc_grad,ker_nonlfunc);   % Need to be updated here!
    else
        %- theta method
        [U1] = burgers1d_fem_newton...
            (opt,x,e_conn,Mass,S,P, ...
            u0,f_FE,f_BE,File_shape,nodes_nondir,bn,...
            ker_nonlfunc_grad,ker_nonlfunc);
        
        %------ add the filter steps
        if filter.op==1
            U1(2:end-1) = (Mass(2:end-1,2:end-1)+...
                filter.delta.^2*S(2:end-1,2:end-1))\(Mass(2:end-1,2:end-1)*U1(2:end-1));
        end
        
        
        
    end
    u0=U1;
    
    num_temp = (time - t_initial)/domain.tsave;
    if ( abs(num_temp - round(num_temp))< 10^(-8))
        count_s = count_s + 1;
        u_save(:,count_s) = U1;
    end
    if f_time_dep~=0
        f_FE = f_BE;
    end
    
end
cputime= toc(time_start);
fprintf(1, ['Time elapsed is ',num2str(cputime),'\n']);
%____________________________% postprocess of simulation
%--- save data
if save_data == 1
    save(File_dns, 'u_save', 'x', 'e_conn', 'Mass', 'S',...
        'opt', 'n_gauss', 'u', 'ux', 'f', ...
        'B', 'bn', 'Len','Nonlfunc','nodes_nondir',...
        'domain','cputime','File_shape','File_smm')
    fprintf(1, ['Data has been stored to ', char(File_dns), '\n']);
end
%--- Output to screen
fprintf(1, ['Time elapsed for simulations is ', num2str(cputime), '\n']);
if fig_evo_p == 1 % output process of evolution to screen
    figure(fig_num),
    for i = 1:size(u_save,2)
        x_cord = 1:length(x);
        plot( x(x_cord), u_save(x_cord,i)),
        pause(.1);
    end
    fig_num = fig_num + 1;
end
if fig_evo == 1
    skip = 1;
    [XX,TT]=meshgrid(x, t_initial:domain.tsave:t_final);
    figure(fig_num)
    mesh(XX, TT, u_save');
    xlabel('x'); ylabel('t');
    %print('-depsc', File_fig)
    fig_num = fig_num + 1;
end
%--- calculate errors and output to screen
if cal_err == 1
    Err_max    = zeros(n_save, 1);       Err_L2 = Err_max;
    Err_H1_semi= Err_max;                Err_H1 = Err_max;
    for i = 1: domain.calerr_skip: n_save
        U1 = u_save(:,i);
        time = t_initial+ (i-1)*domain.tsave*domain.calerr_skip;
        [Err_max(i), Err_L2(i), Err_H1_semi(i), Err_H1(i)] = FEM_oned_error...
            (x, e_conn, U1, u, ux, time, File_shape);
        if i==n_save
            err_max_avg   = sqrt( sum(Err_max.^2)/n_save );
            err_L2_avg    = sqrt( sum(Err_L2.^2)/n_save );
            err_H1_semi_avg = sqrt( sum(Err_H1_semi.^2)/n_save );
            err_H1_avg    = sqrt( sum(Err_H1.^2)/n_save );
            fprintf(1, ['Time average C0 error square is ', num2str(err_max_avg), '\n']);
            fprintf(1, ['Time average L2 error square is ', num2str(err_L2_avg), '\n']);
            fprintf(1, ['Time average H1 error square is ', num2str(err_H1_avg), '\n']);
        end
    end
end
fprintf(1,'===== Last run ends =====\n');
