function [Trilinear1, Trilinear2, Trilinear3]= fem1d_assemble_trilinear_fn...
    (x, e_conn, n_gauss, u, filename_shape)
%=====================================
% 10/31/2011
% Developed by Zhu Wang basing on John Burkdart's codes
% To evaluate the trilinear term
%
% Inputs:
%       x - coordinates matrix
%       e_conn - connective matrix
%       n_gauss -  # of Gauss points
%       u - state variable vector
% Outputs:
%       Trilinear1 (sparse) - (h*(u-0.1)*(1-u), h)
%       Trilinear2 (sparse) - (h*u*(1-u), h)
%       Trilinear3 (sparse) - (h*u*(u-0.1), h)
%======================================

[N,          dim    ] = size(x     );
[n_elements, nel_dof] = size(e_conn);
%--- initialization
load(filename_shape,'W_w_g','W_phi')

% Due to memory limitations, we will fill the mass matrix by integrating
% over partitions of elements.  The constant (max_elem_per_partition)
% should be adjusted based on available memory.
max_elem_per_partition = 5000;

n_part = floor( n_elements / (max_elem_per_partition+1) ) + 1;
elem_segment = floor( linspace(0,n_elements,n_part+1) );
max_part_size = max( diff( elem_segment ) );

if (dim==1)
    [r,w] = FEM_oned_gauss(n_gauss);
    one = ones(size(w));
    for n_pt = 1:n_part
        II = zeros( max_part_size*nel_dof^2,1 );
        JJ = zeros( max_part_size*nel_dof^2,1 );
        XX = zeros( max_part_size*nel_dof^2,1 );
        YY = zeros( max_part_size*nel_dof^2,1 );
        ZZ = zeros( max_part_size*nel_dof^2,1 );
        entry_counter = 0;
        
        for n_el=elem_segment(n_pt)+1:elem_segment(n_pt+1)
            nodes_local            = e_conn(n_el,:);
            w_g     = W_w_g(:,n_el);
            phi     = W_phi(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
            
            u_loc   = u(nodes_local, 1);
            ker1    = (phi*(u_loc-0.1)).*(1-phi*u_loc);
            ker2    = (phi*u_loc).*(1-phi*u_loc);
            ker3    = (phi*u_loc).*(phi*(u_loc-0.1));
            tri_loc1 = FEM_oned_bilinear(ker1, phi, phi, w_g );  %(h*(u-0.1)*(1-u), h)
            if nargout >1
                tri_loc2 = FEM_oned_bilinear(ker2, phi, phi, w_g );  %(h*u*(1-u), h)
            end
            if nargout>2
                tri_loc3 = FEM_oned_bilinear(ker3, phi, phi, w_g );  %(h*u*(u-0.1), h)
            end
            
            %  Assemble into the global system matrix
            for i=1:nel_dof
                for j=1:nel_dof
                    entry_counter = entry_counter + 1;
                    II(entry_counter) = nodes_local(i);
                    JJ(entry_counter) = nodes_local(j);
                    XX(entry_counter) = tri_loc1(i,j);
                    if nargout >1
                        YY(entry_counter) = tri_loc2(i,j);
                    elseif nargout>2
                        ZZ(entry_counter) = tri_loc3(i,j);
                    end
                end
            end
            
        end % n_el loop
        
        if ( n_pt==1 )
            Trilinear1 = sparse( II(1:entry_counter), JJ(1:entry_counter),...
                XX(1:entry_counter),...
                N, N );
            if nargout>1
                Trilinear2 = sparse( II(1:entry_counter), JJ(1:entry_counter),...
                    YY(1:entry_counter),...
                    N, N );
            end
            if nargout>2
                Trilinear3 = sparse( II(1:entry_counter), JJ(1:entry_counter),...
                    ZZ(1:entry_counter),...
                    N, N );
            end
        else
            Trilinear1 = Trilinear1 + ...
                sparse( II(1:entry_counter), JJ(1:entry_counter),...
                XX(1:entry_counter),...
                N, N );
            if nargout>1
                Trilinear2 = Trilinear2 + ...
                    sparse( II(1:entry_counter), JJ(1:entry_counter),...
                    YY(1:entry_counter),...
                    N, N );
            end
            if nargout>2
                Trilinear3 = Trilinear3 + ...
                    sparse( II(1:entry_counter), JJ(1:entry_counter),...
                    ZZ(1:entry_counter),...
                    N, N );
            end
        end
        
    end % n_pt loop
else
    fprintf(1, 'This function only supports 1D cases \n');
end
